
import { logger } from "../../logger";
import { JSONDatabase } from "../ADatabase";
import {ISettings} from "../../Interfaces/ISettings"
export const settings = new (class Settings
    extends JSONDatabase
    implements ISettings
{
    logLevel: "debug" | "info" | "error" = "debug";
    pararels: number = 3;
    mainCrypto: "TRX" = "TRX";
    minimalToWithdraw: number = 0.003;
    maxRefferals: number = 5;
    // to find
    botMessages = {
        verification: "To continue using this bot,",
        tasksSelector: "Choose an option to start earning your TRX",
        taskOver: "Sorry, there are no new ads available.",
        taskComplete: "Success, ",
        notInGroup: "not entered the chat",
        oldMessage: "forwarded is old",
    };
    // to send
    botButtons = {
        earn: "❇️ Earn cryptocurrency",
        balance: "💰 Balance",
        withdraw: "📤 Withdraw",
    };
    // to join
    telegramLinks = {
        botLink: "@hkearn_trx_bot",
        groupsToJoin: ["@hkearn_transactions", "@hkearn_updates"],
    };
    sleepTime = {
        afterDoing: [3600, 7200],
        betweenSessions: [35, 70],
    };
    shuffleAccounts: boolean = true;
    bypassMode: boolean = false;

    constructor() {
        super("settings.json");
    }
    async load(): Promise<void> {
        await super.load();
        const settings = await this.rawDB.getObject<ISettings>("/");

        const defaults = {};
        Object.getOwnPropertyNames(this).forEach((name) => {
            if (name != "rawDB" && name != "isLoaded") {
                defaults[name] = this[name];
            }
        });

        if (Object.keys(settings).length == 0) {
            logger.warn("Setup config first...");
            await this.rawDB.push("/", defaults, true);
            await this.rawDB.save();
        } else {
            mainLoop: for (const setting of Object.keys(defaults)) {
                if (
                    settings[setting] === undefined ||
                    typeof settings[setting] != typeof defaults[setting]
                ) {
                    this[setting] = defaults[setting];
                    logger.warn(
                        `Setting '${setting}' corrupted or undefined. Check out it...`
                    );
                } else {
                    if (typeof settings[setting] == "object") {
                        for (const attr of Object.keys(defaults[setting])) {
                            if (
                                Object.keys(settings[setting]).indexOf(attr) ==
                                -1
                            ) {
                                this[setting] = defaults[setting];
                                logger.warn(
                                    `Setting '${setting}' corrupted. Check out it...`
                                );
                                continue mainLoop;
                            }
                        }
                    }
                    this[setting] = settings[setting];
                }
            }
            logger.level = this.logLevel;
        }
    }
})();