import Docker from "dockerode";
import cliProgress from "cli-progress";
import { settings } from "../Databases/JSONDatabases/Settings";
import { sleep } from "telegram/Helpers";
import { logger } from "../logger";

export const dockerWrapper = new class DockerController {
    containers: Docker.Container[] = [];
    protected multibar = new cliProgress.MultiBar({
        hideCursor: true,
        barsize: 50,
        barCompleteChar: "❌",
        barIncompleteChar: "✔️",
        clearOnComplete: true,
        format: `{status} | {bar} | {value}/{total}sec timeout`,
    });
    async closeOpenedDockers() {
        logger.debug("Closing opened docker containers ");
        for (const container of this.containers) {
        
            try {
                await container.remove({ force: true, v: true });
            } catch (err) { }

        }
        this.containers=[];
    }
    async createTorDocker(proxyPort: number, socksPort: number, controlPort: number, timeout = 200): Promise<Docker.Container> {
        timeout *= 2;
        const docker = new Docker({ socketPath: "/var/run/docker.sock" });
        let isPulled = false;
        mainLoop: for (const image of await docker.listImages()) {
            const tags = image.RepoTags;
            if (tags !== undefined)
                for (const repoTag of tags) {
                    if (repoTag.search("dperson/torproxy:latest") != -1) {
                        isPulled = true;
                        break mainLoop;
                    }
                }
        }
        if (isPulled != true) {
            await docker.pull("dperson/torproxy:latest");
            logger.warn('Docker image pulled.You must restart script...');
            process.exit();
        }

        const options: Docker.ContainerCreateOptions = {
            Image: "dperson/torproxy:latest",
            Env: ["PASSWORD=qwerty", "LOCATION=US", "TOR_NewCircuitPeriod=50"],
            ExposedPorts: {},
            HostConfig: {
                PortBindings: {
                    "8118/tcp": [
                        {
                            HostPort: `${proxyPort}`,
                        },
                    ],
                    "9050/tcp": [
                        {
                            HostPort: `${socksPort}`,
                        },
                    ],
                    "9051/tcp": [
                        {
                            HostPort: `${controlPort}`,
                        },
                    ],
                },
            },
            Healthcheck: {
                Interval: 2 * 1_000_000_000,
                Timeout: 50 * 1_000_000_000,
                StartPeriod: 50 * 1_000_000_000,
                // Test:['CMD_SHELL',`curl -sx localhost:8118 'https://check.torproject.org/' | grep -qm1 Congratulations`]
            },
        };
        const container = await docker.createContainer(options);
        try {
            await container.start();
        } catch (e) {
            logger.error(`Some error when starting docker container\n${e}`)
            throw e
        }

        const progressBar = this.multibar.create(timeout / 2, 0, {
            status: `Starting docker${settings.logLevel == "debug"
                    ? ` ${proxyPort} ${socksPort} ${controlPort}`
                    : ""
                }`,
        });
        while ((await container.inspect())!.State.Health!.Status != "healthy") {
            const state = (await container.inspect())!.State.Health!.Status;
            if (progressBar.getProgress() >= timeout) {
                await container.remove({ force: true, v: true });
                progressBar.update(timeout / 2, { status: "Failed" });
                progressBar.stop();
                throw new Error(`Docker ${container.id} is broken`);
            }
            if (state == "unhealthy") {
                await container.remove({ force: true, v: true });
                progressBar.update(timeout / 2, {
                    status: "Docker is unhealthy...",
                });
                progressBar.stop();
                return await this.createTorDocker(
                    proxyPort,
                    socksPort,
                    controlPort,
                    timeout / 2
                );
            }
            progressBar.increment();
            await sleep(2000);
        }
        progressBar.update(0, { status: "Started" });
        progressBar.stop();

        this.containers.push(container);
        return container;
    }
}